/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "kivio_ruler.h"

#include <kapp.h>
#include <qglobal.h>

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>

#define MARKER_WIDTH 6
#define MARKER_HEIGHT 3

#define RULER_SIZE 20

KivioRuler::KivioRuler(Orientation o, MeasurementUnit mu, QWidget *parent, const char *name)
: QFrame (parent, name)
{
  setFrameStyle (Box | Raised);
  setLineWidth (1);
  setMidLineWidth (0);
  orientation = o;
  munit = mu;
  m_pZoom = 100;
  firstVisible = 0;
  buffer = 0L;
  currentPosition = -1;

  if (orientation == Horizontal) {
    setFixedHeight (RULER_SIZE);
    initMarker (MARKER_WIDTH, MARKER_HEIGHT);
  } else {
    setFixedWidth (RULER_SIZE);
    initMarker (MARKER_HEIGHT, MARKER_WIDTH);
  }

  const char* nums[] = {
    "70 7 2 1",
    "  c Black",
    "X c None",
    "XX   XXXXXX XXXX   XXXX   XXXXXX XXX     XXXX  XXX     XXX   XXXX   XX",
    "X XXX XXXX  XXX XXX XX XXX XXXX  XXX XXXXXXX XXXXXXXXX XX XXX XX XXX X",
    "X XXX XXXXX XXXXXXX XXXXXX XXX X XXX XXXXXX XXXXXXXXX XXX XXX XX XXX X",
    "X XXX XXXXX XXXXX  XXXXX  XXX XX XXX    XXX    XXXXXX XXXX   XXXX    X",
    "X XXX XXXXX XXXX XXXXXXXXX XX     XXXXXX XX XXX XXXX XXXX XXX XXXXXX X",
    "X XXX XXXXX XXX XXXXXX XXX XXXXX XXXXXXX XX XXX XXXX XXXX XXX XXXXX XX",
    "XX   XXXXXX XXX     XXX   XXXXXX XXX    XXXX   XXXXX XXXXX   XXXX  XXX"
    };
  m_pNums = new QPixmap(nums);
}

void KivioRuler::initMarker (int w, int h)
{
  QPainter p;
  QPointArray pts (3);
  static QCOORD hpoints [] = { 0,0, MARKER_WIDTH-1,0,
			       (MARKER_WIDTH-1)/2,MARKER_HEIGHT-1 };
  static QCOORD vpoints [] = { 0,0, MARKER_HEIGHT-1,(MARKER_WIDTH-1)/2,
			       0,MARKER_WIDTH-1 };

  if (orientation == Horizontal)
    pts.putPoints (0, 3, hpoints);
  else
    pts.putPoints (0, 3, vpoints);

  marker = new QPixmap (w, h);
  p.begin (marker);
  p.setPen (black);
  p.setBrush (black);
  p.setBackgroundColor (colorGroup ().background ());
  p.eraseRect (0, 0, w, h);
  p.drawPolygon (pts);
  p.end ();

  bg = new QPixmap (w, h);
  p.begin (bg);
  p.setBackgroundColor (colorGroup ().background ());
  p.eraseRect (0, 0, w, h);
  p.end ();
}

void KivioRuler::recalculateSize( QResizeEvent* )
{
  if (buffer) {
    delete buffer;
    buffer = 0L;
  }

  int w, h;

  if (orientation == Horizontal) {
    w = width();
    h = RULER_SIZE;
  } else {
    w = RULER_SIZE;
    h = height();
  }
  buffer = new QPixmap(w, h);
  drawKivioRuler();
  updatePointer(currentPosition,currentPosition);
}

MeasurementUnit KivioRuler::measurementUnit() const
{
  return  munit;
}

void KivioRuler::setMeasurementUnit( MeasurementUnit mu )
{
  munit = mu;
  drawKivioRuler();
  updatePointer(currentPosition,currentPosition);
  repaint();
}

void KivioRuler::setZoom( int zoom )
{
  m_pZoom = zoom;
  recalculateSize(0L);
  drawKivioRuler();
  updatePointer(currentPosition, currentPosition);
  repaint();
}

void KivioRuler::updatePointer (int x, int y )
{
  if (! buffer)
    return;

  QRect r1, r2;
  int pos = 0;

  if (orientation == Horizontal) {
    if (currentPosition != -1) {
      pos = qRound (currentPosition * m_pZoom / 100.0 - (firstVisible >= 0 ? 0 : firstVisible) - MARKER_WIDTH / 2);
      r1 = QRect (pos - (firstVisible >= 0 ? firstVisible : 0), 1, MARKER_WIDTH, MARKER_HEIGHT);
      bitBlt (buffer, pos, 1, bg, 0, 0, MARKER_WIDTH, MARKER_HEIGHT);
    }
    if (x != -1) {
      pos = qRound (x * m_pZoom / 100.0 - (firstVisible >= 0 ? 0 : firstVisible) - MARKER_WIDTH / 2);
      r2 = QRect (pos - (firstVisible >= 0 ? firstVisible : 0), 1, MARKER_WIDTH, MARKER_HEIGHT);
      bitBlt (buffer, pos, 1, marker, 0, 0, MARKER_WIDTH, MARKER_HEIGHT);
      currentPosition = x;
    }
  } else {
    if (currentPosition != -1) {
      pos = qRound (currentPosition * m_pZoom / 100.0 - (firstVisible >= 0 ? 0 : firstVisible) - MARKER_HEIGHT / 2);
      r1 = QRect (1, pos - (firstVisible >= 0 ? firstVisible : 0), MARKER_HEIGHT, MARKER_WIDTH);
      bitBlt (buffer, 1, pos, bg, 0, 0, MARKER_HEIGHT, MARKER_WIDTH);
    }
    if (y != -1) {
      pos = qRound (y * m_pZoom / 100.0 - (firstVisible >= 0 ? 0 : firstVisible) - MARKER_HEIGHT / 2);
      r2 = QRect (1, pos -  (firstVisible >= 0 ? firstVisible : 0), MARKER_HEIGHT, MARKER_WIDTH);
      bitBlt (buffer, 1, pos, marker, 0, 0, MARKER_HEIGHT, MARKER_WIDTH);
      currentPosition = y;
    }
  }
  repaint(r1.unite(r2));
}

void KivioRuler::updateVisibleArea( int xpos, int ypos )
{
  if (orientation == Horizontal)
    firstVisible = xpos;
  else
    firstVisible = ypos;

  drawKivioRuler();
  repaint();
}

void KivioRuler::paintEvent( QPaintEvent* e )
{
  if (!buffer)
    return;

  const QRect& rect = e->rect ();

  if (orientation == Horizontal) {
      bitBlt (this, rect.x (), rect.y (), buffer,
	      rect.x (), rect.y (),
	      rect.width (), rect.height ());
  } else {
      bitBlt (this, rect.x (), rect.y (), buffer,
	      rect.x (), rect.y (),
	      rect.width (), rect.height ());
  }
  QFrame::paintEvent (e);
}

void KivioRuler::drawKivioRuler()
{
  QPainter p;
  QString buf;

  if (!buffer)
    return;

  p.begin(buffer);
  p.setPen(QColor(0x70,0x70,0x70));
  p.setBackgroundColor(colorGroup().background());
  buffer->fill(backgroundColor());

  if (orientation == Horizontal) {
    switch (munit) {
      case UnitPoint: {
        break; }
/*************************************************************/
      case UnitMillimeter: {

        int pos = 0;
        bool s1 = cvtMmToPt(1)*(m_pZoom/100.0) > 3.0;
        bool s2 = cvtMmToPt(5)*(m_pZoom/100.0) > 3.0;
        bool s3 = cvtMmToPt(10)*(m_pZoom/100.0) > 3.0;

        float cxmm = cvtPtToMm(7*4)/(m_pZoom/100.0);
        int step = ((int)(cxmm/5.0)+1)*5;
        int mmStart = (int)(cvtPtToMm(firstVisible)/(m_pZoom/100.0));

        do {
          pos = (int)(cvtMmToPt(mmStart)*(m_pZoom/100.0) - firstVisible);

          if ( s3 && mmStart % 10 == 0 )
            p.drawLine(pos,RULER_SIZE-9,pos,RULER_SIZE);

          if ( s2 && mmStart % 5 == 0 )
            p.drawLine(pos,RULER_SIZE-7,pos,RULER_SIZE);

          if ( s1 )
            p.drawLine(pos,RULER_SIZE-5,pos,RULER_SIZE);

          if ( mmStart % step == 0 ) {
            buf.setNum(abs(mmStart));
            drawNums(&p,pos,4,buf,true);
          }

          mmStart++;
        } while (pos < buffer->width());

        break; }
/*************************************************************/
      case UnitCentimeter: {
        break; }
      case UnitDidot: {
        break; }
      case UnitCicero: {
        break; }
      case UnitPica: {
        break; }
      case UnitInch: {
        break; }
      default:
        break;
    }
  } else {
    switch (munit) {
      case UnitPoint: {
        break; }
      case UnitDidot: {
        break; }
/*************************************************************/
      case UnitMillimeter: {

        int pos = 0;
        bool s1 = cvtMmToPt(1)*(m_pZoom/100.0) > 3.0;
        bool s2 = cvtMmToPt(5)*(m_pZoom/100.0) > 3.0;
        bool s3 = cvtMmToPt(10)*(m_pZoom/100.0) > 3.0;

        float cxmm = cvtPtToMm(8*4)/(m_pZoom/100.0);
        int step = ((int)(cxmm/5.0)+1)*5;
        int mmStart = (int)(cvtPtToMm(firstVisible)/(m_pZoom/100.0));

        do {
          pos = (int)(cvtMmToPt(mmStart)*(m_pZoom/100.0) - firstVisible);

          if ( s3 && mmStart % 10 == 0 )
            p.drawLine(RULER_SIZE-9,pos,RULER_SIZE,pos);

          if ( s2 && mmStart % 5 == 0 )
            p.drawLine(RULER_SIZE-7,pos,RULER_SIZE,pos);

          if ( s1 )
            p.drawLine(RULER_SIZE-5,pos,RULER_SIZE,pos);

          if ( mmStart % step == 0 ) {
            buf.setNum(abs(mmStart));
            drawNums(&p,4,pos,buf,false);
          }

          mmStart++;
        } while (pos < buffer->height());

        break; }
/*************************************************************/
      case UnitCentimeter: {
        break; }
      case UnitCicero: {
        break; }
      case UnitPica: {
        break; }
      case UnitInch: {
        break; }
      default:
        break;
    }
  }
  p.end();
}

void KivioRuler::resizeEvent ( QResizeEvent* e )
{
  recalculateSize(e);
}

void KivioRuler::show()
{
  if (orientation == Horizontal) {
    setFixedHeight(RULER_SIZE);
    initMarker(MARKER_WIDTH,MARKER_HEIGHT);
  } else {
    setFixedWidth(RULER_SIZE);
    initMarker(MARKER_HEIGHT,MARKER_WIDTH);
  }
  QWidget::show();
}

void KivioRuler::hide()
{
  if (orientation == Horizontal)
    setFixedHeight(1);
  else
    setFixedWidth(1);
//  QWidget::hide();
}

void KivioRuler::mousePressEvent( QMouseEvent* )
{
  isMousePressed = true;
}

void KivioRuler::mouseMoveEvent( QMouseEvent* me )
{
   /* Implement the hooks so that a helpline can be drawn out of the ruler:
      - if the mouse is on the widget, draw a helpline
      - if it is outside remove the XOR'd helpline
      - if the mouse it over the page view, set the helpline
       (different place: update the helpline position in the status bar)*/
   if (isMousePressed) {
     emit drawHelpline (me->x () +
		       (orientation == Horizontal ? firstVisible : 0) -
		       RULER_SIZE,
		       me->y () +
		       (orientation == Vertical ? firstVisible : 0) -
		       RULER_SIZE,
		       (orientation==Horizontal) ? true : false );
   }
}

void KivioRuler::mouseReleaseEvent( QMouseEvent* me )
{
  if (isMousePressed) {
     isMousePressed = false;
     emit addHelpline (me->x () + 
		       (orientation == Horizontal ? firstVisible : 0) -
		       RULER_SIZE, 
		       me->y () +
		       (orientation == Vertical ? firstVisible : 0) -
		       RULER_SIZE, 
		       (orientation==Horizontal) ? true : false );
  }
}

KivioRuler::~KivioRuler()
{
//  delete marker;
//  delete bg;
//  delete m_pNums;
}

void KivioRuler::drawNums( QPainter* p, int x, int y, QString& num,  bool orientationHoriz )
{
  if (orientationHoriz)
    x -= 7;
  else
    y -= 8;

  for ( uint k = 0; k < num.length(); k++ ) {
    int st = num.at(k).digitValue()*7;
    p->drawPixmap(x,y,*m_pNums,st,0,7,7);
    if (orientationHoriz)
      x += 7;
    else
      y += 8;
  }
}
